import {
  __objRest
} from "../chunk-G2ADBYYC.js";
import binarySearch from "./utils/utils";
const getStartIndex = ({ state, scrollTop = 0 }) => {
  if (!state || !state.positions)
    return 0;
  return binarySearch(state.positions, scrollTop);
};
const updatePositions = ({ state, items, props }) => () => {
  var _a;
  if (!((_a = items.value) == null ? void 0 : _a.length))
    return;
  const nodes = items.value;
  const positions = state.positions;
  let dValue = 0;
  const attr = props.direction === "vertical" ? "height" : "width";
  nodes.forEach((node) => {
    const s = node.getBoundingClientRect()[attr];
    let index = +node.id.slice(1);
    let oldS = positions[index].height;
    dValue = oldS - s;
    if (!dValue)
      return;
    positions[index].bottom = positions[index].bottom - dValue;
    positions[index].height = s;
    for (let k = index + 1; k < index + nodes.length; k++) {
      positions[k].top = positions[k - 1].bottom;
      positions[k].bottom = positions[k].bottom - dValue;
    }
  });
  if (!dValue || !nodes.length)
    return;
  for (let k = nodes.length; k < positions.length; k++) {
    positions[k].top = positions[k - 1].bottom;
    positions[k].bottom = positions[k].bottom - dValue;
  }
  if (!dValue)
    return;
};
const calculateTotalSize = ({ props, state }) => {
  if (props.itemSize) {
    return () => {
      return props.itemSize * state.data.length;
    };
  }
  return () => {
    var _a;
    const lastPosition = (_a = state.positions) == null ? void 0 : _a[state.positions.length - 1];
    return lastPosition ? lastPosition.bottom : 0;
  };
};
const handleScroll = (_a) => {
  var _b = _a, { props, state, virtualScroll, nextTick, items } = _b, rest = __objRest(_b, ["props", "state", "virtualScroll", "nextTick", "items"]);
  let animationFrameId = null;
  const processScroll = async () => {
    var _a2;
    if (!virtualScroll.value)
      return;
    await nextTick();
    let start, end, totalSize, bufferSize;
    const size = props.visibleSize;
    const viewNum = Math.ceil(size / ((_a2 = props.itemSize) != null ? _a2 : props.estimatedItemSize));
    const scrollPosition = props.direction === "vertical" ? virtualScroll.value.scrollTop : virtualScroll.value.scrollLeft;
    if (state.temporary.prerender) {
      start = 0;
      end = Math.min(props.prerender, state.data.length);
      totalSize = null;
    } else {
      if (props.itemSize) {
        const viewStart = ~~(scrollPosition / props.itemSize);
        const bufferItems = props.buffer * viewNum;
        start = viewStart - bufferItems > 0 ? viewStart - bufferItems : 0;
        end = viewStart + viewNum + bufferItems < state.data.length ? viewStart + viewNum + bufferItems : state.data.length;
        state.visibleData = state.data.slice(start, end);
        state.translate = start * props.itemSize;
        return;
      }
      const startIndex = getStartIndex({ state, scrollTop: scrollPosition });
      const bufferCount = viewNum * props.buffer;
      const endIndex = startIndex + viewNum;
      const aboveCount = Math.min(startIndex, bufferCount);
      const belowCount = Math.min(state.data.length - endIndex, bufferCount);
      start = startIndex - aboveCount;
      end = endIndex + belowCount;
      state.visibleData = state.data.slice(start, end);
      if (startIndex >= 1) {
        bufferSize = state.positions[startIndex].top - (state.positions[start] ? state.positions[start].top : 0);
        state.translate = state.positions[startIndex - 1].bottom - bufferSize;
      } else {
        state.translate = 0;
      }
      await updatePositions({ state, items, props });
      totalSize = calculateTotalSize({ state, props });
    }
    state.totalSize = totalSize;
  };
  return (event) => {
    if (animationFrameId !== null) {
      cancelAnimationFrame(animationFrameId);
    }
    animationFrameId = requestAnimationFrame(() => {
      processScroll();
    });
  };
};
const initPositions = ({ props, state }) => () => {
  state.positions = state.data.map((item, index) => ({
    index,
    top: index * props.estimatedItemSize,
    bottom: (index + 1) * props.estimatedItemSize,
    height: props.estimatedItemSize
  }));
};
const scrollToPosition = ({ virtualScroll, props }) => (position) => {
  const scrollProperty = props.direction === "vertical" ? "scrollTop" : "scrollLeft";
  const viewPort = virtualScroll.value;
  if (viewPort) {
    viewPort[scrollProperty] = position;
  }
};
const scrollToItem = ({ api, props, state }) => (index) => {
  let position;
  if (props.itemSize === null) {
    position = index > 0 ? state.positions[index - 1].bottom : 0;
  } else {
    position = (index - 1) * props.itemSize;
  }
  api.scrollToPosition(position);
};
export {
  calculateTotalSize,
  handleScroll,
  initPositions,
  scrollToItem,
  scrollToPosition,
  updatePositions
};
